 /***********************************************************************\
 * Copyright (c) 2013 by  LLC. All Rights Reserved.                *
 *                                                                       *
 * This is the header file for debug trace related stuff                 *
 \***********************************************************************/

/*
 * HISTORY:
 *
 *    Rev. 1.00 (03/21/13) AK: first release
 *
 */

#if !defined(___HB_TRACE_H__)
#define ___HB_TRACE_H__

//#include "hb_build_config.h"           // HB_TRACE
#include "os_dependency.h"             // HB_DOUBLE


#ifdef HB_TRACE

    uint32_t hb_dbg_getmask(void);
    uint32_t hb_dbg_getmask2(void);
    void hb_dbg_setmask(uint32_t new_mask);
    void hb_dbg_setmask2(uint32_t new_mask);

    void hb_print_buf(char* pBuf, int len);
    void hb_print_buf_as_hex(char* pBuf, int len);
    void hb_print_buf_as_hex2(char* pBuf, int len);

    void printDouble(HB_DOUBLE v, uint32_t decDigits);

    #define DBG_VAR(x) x
    #define EXECUTE_IF(mask, q)   if (hb_dbg_getmask() & mask) {q}
    #define EXECUTE_IF2(mask, q)  if (hb_dbg_getmask2() & mask) {q}

#else // HB_TRACE

    #define DBG_VAR(x)
    #define EXECUTE_IF(mask, q)
    #define EXECUTE_IF2(mask, q)

#endif // HB_TRACE

#define TRACE_ERROR         "### ERR: "
#define TRACE_ERR           TRACE_ERROR
#define TRACE_WARNING       "### WARN: "
#define TRACE_WARN          TRACE_WARNING
#define TRACE_OK(message)   "[OK]" #message "\n\n"
#define TRACE_FAIL(message) "[FAIL]" #message "\n\n"

#define BIT_00000001    0x00000001
#define BIT_00000002    0x00000002
#define BIT_00000004    0x00000004
#define BIT_00000008    0x00000008

#define BIT_00000010    0x00000010
#define BIT_00000020    0x00000020
#define BIT_00000040    0x00000040
#define BIT_00000080    0x00000080

#define BIT_00000100    0x00000100
#define BIT_00000200    0x00000200
#define BIT_00000400    0x00000400
#define BIT_00000800    0x00000800

#define BIT_00001000    0x00001000
#define BIT_00002000    0x00002000
#define BIT_00004000    0x00004000
#define BIT_00008000    0x00008000

#define BIT_00010000    0x00010000
#define BIT_00020000    0x00020000
#define BIT_00040000    0x00040000
#define BIT_00080000    0x00080000

#define BIT_00100000    0x00100000
#define BIT_00200000    0x00200000
#define BIT_00400000    0x00400000
#define BIT_00800000    0x00800000

#define BIT_01000000    0x01000000
#define BIT_02000000    0x02000000
#define BIT_04000000    0x04000000
#define BIT_08000000    0x08000000

#define BIT_10000000    0x10000000
#define BIT_20000000    0x20000000
#define BIT_40000000    0x40000000
#define BIT_80000000    0x80000000

/* ------------------------------------------------ */

#define DBG_ERR(q)     EXECUTE_IF(BIT_00000001, q)
#define DBG_WARN(q)    EXECUTE_IF(BIT_00000002, q)
#define DBG_INIT(q)    EXECUTE_IF(BIT_00000004, q)
#define DBG_INFO(q)    EXECUTE_IF(BIT_00000008, q)

#define DBG_STOR(q)    EXECUTE_IF(BIT_00010000, q)
#define DBG_STOR2(q)   EXECUTE_IF(BIT_00020000, q)

#define DBG_IPC(q)     EXECUTE_IF(BIT_10000000, q)
#define DBG_TEST(q)    EXECUTE_IF(BIT_20000000, q)
#define DBG_UTEST(q)   EXECUTE_IF(BIT_40000000, q)
#define DBG_CTRL(q)    EXECUTE_IF(BIT_80000000, q)

#ifdef HB_FACTORY_TEST
#define DBG_TEST2(q)   EXECUTE_IF(BIT_40000000, q)
#endif
/* ------------------------------------------------ */

#define DBG2_BIT1(q)   EXECUTE_IF2(BIT_00000001, q)
#define DBG2_BIT2(q)   EXECUTE_IF2(BIT_00000002, q)
#define DBG2_BIT3(q)   EXECUTE_IF2(BIT_00000004, q)
#define DBG2_BIT4(q)   EXECUTE_IF2(BIT_00000008, q)
/* ------------------------------------------------ */


#define TRACE00(s)                     HBPRINT((s))
#define TRACE01(s,a)                   HBPRINT((s),a)
#define TRACE02(s,a1,a2)               HBPRINT((s),a1,a2)
#define TRACE03(s,a1,a2,a3)            HBPRINT((s),a1,a2,a3)
#define TRACE04(s,a1,a2,a3,a4)         HBPRINT((s),a1,a2,a3,a4)
#define TRACE05(s,a1,a2,a3,a4,a5)      HBPRINT((s),a1,a2,a3,a4,a5)
#define TRACE06(s,a1,a2,a3,a4,a5,a6)   HBPRINT((s),a1,a2,a3,a4,a5,a6)
/* ------------------------------------------------ */

#ifdef DEBUG_OUTPUT_INTO_FILE

extern char dbg_buf[];
extern void save_dbg_buf(uint32_t len);

#define DBG_FILE_NAME   "0:debug.txt"
#define DBG_FILE_SIZE    0
#include "Core/hb_storage.h"

extern uint32_t _create_dbg_file(uint32_t size);
extern uint32_t dbg_file_inited;
extern f_handle_t dbg_file;

#ifdef HBPRINT
#undef HBPRINT
#endif

#define HBPRINT(fmt,...) do {                                \
    uint32_t msg_len = sprintf(dbg_buf, fmt, ##__VA_ARGS__); \
    printf(fmt, ##__VA_ARGS__);                              \
    save_dbg_buf(msg_len);                                   \
} while (0);

#define FPRINT  HBPRINT
#define FPRINT1 HBPRINT
#define FPRINT2 HBPRINT
#define FPRINT3 HBPRINT
#define FPRINT4 HBPRINT

#endif // DEBUG_OUTPUT_INTO_FILE


#ifdef DEBUG_OUTPUT_MEMORY

extern void init_inmemory_logger();
#define LOGAREA_SIZE 32768
extern char _logarea[LOGAREA_SIZE];

#endif // DEBUG_OUTPUT_MEMORY

#endif  // ___HB_TRACE_H__
