
#include "data_log_test.h"

#ifdef HB_UNIT_TESTS

#include "Core/hb_hal.h"
#include "Core/hb_data_log.h"

#include "Utils/hb_utils.h"

    #define max(a,b) \
        ({ __typeof__ (a) _a = (a); \
            __typeof__ (b) _b = (b); \
            _a > _b ? _a : _b; })

    #define min(a,b) \
        ({ __typeof__ (a) _a1 = (a); \
            __typeof__ (b) _b1 = (b); \
            _a1 < _b1 ? _a1 : _b1; })


#define _BUF_SIZE    11

uint8_t _buf[_BUF_SIZE] = {0};

/* --- test HB_DATA_LOG ------------------------------------ */
#define TEST_LOG_NAME               "test#00"
#define TEST_DATA_REC_SIZE          _BUF_SIZE
#define TEST_MAX_DATA_RECS_TO_STORE 100

static HB_DATA_LOG _test_logs[MAX_DATA_FILE_ID];

static HB_DATA_LOG* _p_test_log = NULL;

static uint8_t _log_name[MAX_DATA_FILE_ID][32];
/* --------------------------------------------------------- */

/* --- test string ----------------------------------------- */
uint8_t*  test_str[] = {
    (uint8_t*)"DEED BEER_",
    (uint8_t*)"deed beer_",
    (uint8_t*)" 123234345",
    (uint8_t*)"test save_",
};
/* --------------------------------------------------------- */

#define SOME_VALUE  55

// BV TODO: use rand() instead of static test string

#if defined(EMULATE_TARGET_FS)

static uint32_t _test_cases_count = 0;

void print_module_name(const char* pModuleName)
{
    _test_cases_count = 0;

    HBPRINT("\n");
    HBPRINT("*******************************************************\n");
    HBPRINT("* Unit test for module '%s'\n", pModuleName);
    HBPRINT("*******************************************************\n");
    HBPRINT("\n");
}

uint32_t run_test_case(const char*pTestCaseName, test_case_func_t pTestCaseFunc)
{
    uint32_t status;

    HBPRINT("  -----------------------------------------------------\n");
    HBPRINT("  Test Case: %s\n", pTestCaseName);
    HBPRINT("  -----------------------------------------------------\n");
    HBPRINT("\n");

    status = (pTestCaseFunc)();

    if (status == HB_SUCCESS) {
        HBPRINT("  Passed\n");
    } else {
        HBPRINT("  FAILED!!!\n");
    }
    HBPRINT("  -----------------------------------------------------\n");
    HBPRINT("\n\n");

    _test_cases_count++;
    return status;
}

void print_status(uint32_t status)
{
    HBPRINT("\n");
    if (status == HB_SUCCESS) {
        HBPRINT("Test finished OK!\n");
    } else {
        HBPRINT("Test status: %ld out of %ld test cases FAILED! Test FAILED!\n", status, _test_cases_count);
    }
    HBPRINT("*******************************************************\n\n\n");
}
#endif

void _print_current_status(uint32_t status);

uint32_t _test_write_read(uint32_t* p_rec_count);
uint32_t _test_count_and_size(uint32_t* p_rec_count);
uint32_t _test_read_raw_data_and_close(uint32_t* p_rec_count);

void _fill_test_log_struct(void);
uint32_t _disk_init(void);
uint32_t _test_log_init(void);
uint32_t test_data_log_interface(void);
uint32_t test_data_log_huge_records(void);
uint32_t _test_cur_dat_log(void);

uint32_t _check_Nth_cycle(void);


uint32_t run_data_log_unit_test(void)
{
    uint32_t rc = HB_SUCCESS;

    print_module_name("DataLog");

    _fill_test_log_struct();

    rc = _disk_init();
    if (rc != 0) {
        HBPRINT("ERROR: Can't run Data Log to test\n");
        return HB_FAIL;
    }
    HBPRINT("\n");

    // BV TODO: print data log metadata info

    /* ----- Test cases ---------------------------------------- */
    //rc += run_test_case("DataLog Interface", test_data_log_interface);

    rc += run_test_case("DataLog at big amount of records", test_data_log_huge_records);
    /* --------------------------------------------------------- */

    print_status(rc);
    return rc;
}


void _fill_test_log_struct(void)
{
    uint32_t i = 0;

    hal_memset(_test_logs, 0, sizeof(_test_logs));
    hal_memset(_log_name, 0, sizeof(_log_name));

    for (i = 0; i < MAX_DATA_FILE_ID; i++)
    {
        _test_logs[i].log_name = (const char*)&_log_name[i][0];
        *(uint32_t*)&_test_logs[i].file_id = i;
        *(uint32_t*)&_test_logs[i].rec_size = TEST_DATA_REC_SIZE;
        *(uint32_t*)&_test_logs[i].max_recs = TEST_MAX_DATA_RECS_TO_STORE;
        *(uint32_t*)&_test_logs[i].read_file_index = -1;

        hal_memcpy(&_log_name[i][0], TEST_LOG_NAME, 5);
        int2ascii(i, (uint8_t*)(&_log_name[i][5]));
    }

}

uint32_t _disk_init(void)
{
    hal_turn_on_SD_card();

    hb_stor_format_drive(HB_FALSE);

    if (hb_storage_init() != HB_SUCCESS)
    {
        HBPRINT("\n### ERR: SD-card init failed!!!\n");
        return HB_FAIL;
    }
    else
    {
        DBG_UTEST(HBPRINT("\nSD-card init OK\n");)
    }

    return HB_SUCCESS;
}


uint32_t test_data_log_interface(void)
{
    uint32_t rc = HB_SUCCESS;
    uint32_t i = 0;

    // delete .cfg-file for clear start
#if defined(HB_ITS_ONLY) || (defined(HB_WIN32) && defined(HB_UNIT_TESTS))
    hb_data_log_remove_cfg();
#endif
    hb_storage_uninit();
    hb_stor_format_drive(0);
    hb_storage_init();

    DBG_UTEST(HBPRINT("\nInit Data Log...\n");)
    rc += hb_data_log_init();
    DBG_UTEST(HBPRINT("Init Data Log is");)
    _print_current_status(rc);

    DBG_UTEST(HBPRINT("Check init recs_counts table...");)
    for (i = 0; i < MAX_DATA_FILE_ID; i++)
    {
        _p_test_log = &_test_logs[i];

        if (hb_data_log_get_num_records(_p_test_log) != 0) {
            rc += HB_FAIL;
            break;
        }
    }
    _print_current_status(rc);

    hal_feed_the_dog();
    DBG_UTEST(HBPRINT("\nCheck interface for corner cases:");)
    DBG_UTEST(HBPRINT("\n*********************************************************************\n");)
    for (i = 0; i < 1/*MAX_DATA_FILE_ID*/; i++)
    {
        _p_test_log = &_test_logs[i];
        hb_data_log_delete(_p_test_log);

        rc += _check_Nth_cycle();

        DBG_UTEST(HBPRINT("\nCheck deleting .dat-file...\n");)
        hb_data_log_delete(_p_test_log);
        rc += (hb_data_log_open(_p_test_log) == DF_STATUS_OK) ? HB_FAIL : 0;
        DBG_UTEST(HBPRINT("Check deleting .dat-file is");)
        _print_current_status(rc);
    }
    DBG_UTEST(HBPRINT("\n*********************************************************************\n");)
    DBG_UTEST(HBPRINT("Check interface for corner cases is ");)
    _print_current_status(rc);
    if (rc) {
        return HB_FAIL;
    }

    hal_feed_the_dog();
    DBG_UTEST(HBPRINT("\nCheck interface at all closed log files:");)
    DBG_UTEST(HBPRINT("\n*********************************************************************\n");)
    for (i = 0; i < MAX_DATA_FILE_ID; i++)
    {
        _p_test_log = &_test_logs[i];

        rc += _test_cur_dat_log();

        DBG_UTEST(HBPRINT("\nCheck deleting .dat-file...\n");)
        hb_data_log_delete(_p_test_log);
        rc += (hb_data_log_open(_p_test_log) == DF_STATUS_OK) ? HB_FAIL : 0;
        DBG_UTEST(HBPRINT("Check deleting .dat-file is");)
        _print_current_status(rc);
    }
    DBG_UTEST(HBPRINT("\n*********************************************************************\n");)
    DBG_UTEST(HBPRINT("Check interface at all closed log files is");)
    _print_current_status(rc);
    if (rc) {
        return HB_FAIL;
    }

    hal_feed_the_dog();
    DBG_UTEST(HBPRINT("\nCheck interface at some open log files:");)
    DBG_UTEST(HBPRINT("\n*********************************************************************\n");)
    // open some .dat-files
    for (i = 4; i < 7; i++)
    {
        _p_test_log = &_test_logs[i];

        _test_log_init();
    }
    hal_feed_the_dog();
    for (i = 0; i < MAX_DATA_FILE_ID; i++)
    {
        _p_test_log = &_test_logs[i];

        if ((i >= 4) && (i < 7)) {
            hb_data_log_close(_p_test_log);
        }

        rc += _test_cur_dat_log();

        if ((i < 4) || (i >=7)) {
            hb_data_log_delete(_p_test_log);
        } else {
            _test_log_init();
        }

    }
    DBG_UTEST(HBPRINT("\n*********************************************************************\n");)
    DBG_UTEST(HBPRINT("Check interface at some open log files is");)
    _print_current_status(rc);
    if (rc) {
        return HB_FAIL;
    }

    hal_feed_the_dog();
    DBG_UTEST(HBPRINT("\nCheck interface at all open log files:");)
    DBG_UTEST(HBPRINT("\n*********************************************************************\n");)
    // open all .dat-files
    for (i = 0; i < MAX_DATA_FILE_ID; i++)
    {
        _p_test_log = &_test_logs[i];

        hb_data_log_close(_p_test_log);
        _test_log_init();
    }
    hal_feed_the_dog();
    for (i = 0; i < MAX_DATA_FILE_ID; i++)
    {
        _p_test_log = &_test_logs[i];

        hb_data_log_close(_p_test_log);

        rc += _test_cur_dat_log();

        _test_log_init();
    }
    DBG_UTEST(HBPRINT("\n*********************************************************************\n");)
    DBG_UTEST(HBPRINT("Check interface at all open log files is");)
    _print_current_status(rc);
    if (rc) {
        return HB_FAIL;
    }
    hal_feed_the_dog();

    hb_data_log_uninit();
    // BV TODO: add check closing file in hb_data_log_uninit()

    DBG_UTEST(HBPRINT("\n");)

    return rc;
}

uint32_t _test_cur_dat_log(void)
{
    uint32_t rc = HB_SUCCESS, rc_all = HB_SUCCESS;
    uint32_t rec_count = 0;

    DBG_UTEST(HBPRINT("\n  Test DataLog interface at file#%ld:\n", _p_test_log->file_id);)

    hb_data_log_delete(_p_test_log); // for clear start

    rc = HB_SUCCESS;
    DBG_UTEST(HBPRINT("Init Test .dat-file...\n");)
    rc += _test_log_init();
    DBG_UTEST(HBPRINT("Init Test .dat-file is");)
    _print_current_status(rc);
    rc_all += rc;

    rc = HB_SUCCESS;
    DBG_UTEST(HBPRINT("\n  Check interface at empty log:\n");)
    rc += _test_write_read(&rec_count);
    rc += _test_count_and_size(&rec_count);
    rc += _test_read_raw_data_and_close(&rec_count);
    DBG_UTEST(HBPRINT("\n  Check interface at empty log is\n");)
    _print_current_status(rc);
    rc_all += rc;

    rc = HB_SUCCESS;
    DBG_UTEST(HBPRINT("\n  Check interface at not empty log:\n");)
    rc += _test_log_init();
    rc += _test_write_read(&rec_count);
    rc += _test_count_and_size(&rec_count);
    rc += _test_read_raw_data_and_close(&rec_count);
    DBG_UTEST(HBPRINT("\n  Check interface at not empty log is\n");)
    _print_current_status(rc);
    rc_all += rc;

    return rc_all;
}


uint32_t _test_log_init(void)
{
    /* trying opening log file */
    if (hb_data_log_open(_p_test_log) != DF_STATUS_OK)
    {
        /* if we can't open an existing file, we try re-creating it */
        if (hb_data_log_create(_p_test_log) != DF_STATUS_OK) {
            /* if we can't even re-create the log file(s), we can't go on */
            return HB_FAIL;
        }
    }

    return HB_SUCCESS;
}

uint32_t _test_write_read(uint32_t* p_rec_count)
{
    uint32_t rc = HB_SUCCESS, rc_all = HB_SUCCESS;
    uint32_t i = 0;
    uint32_t rec_count = *p_rec_count;

    rc = HB_SUCCESS;
    DBG_UTEST(HBPRINT("Add 1st record...");)
    rc += hb_data_log_add_record(_p_test_log, test_str[0]);
    rc += hb_data_log_read_record(_p_test_log, rec_count, _buf);
    rc += hal_memcmp(_buf, test_str[0], _BUF_SIZE);
    _print_current_status(rc);
    rec_count++;
    rc_all += rc;

    rc = HB_SUCCESS;
    DBG_UTEST(HBPRINT("Add 2nd record...");)
    rc += hb_data_log_add_record(_p_test_log, test_str[1]);
    rc += hb_data_log_read_record(_p_test_log, rec_count, _buf);
    rc += hal_memcmp(_buf, test_str[1], _BUF_SIZE);
    _print_current_status(rc);
    rec_count++;
    rc_all += rc;

    rc = HB_SUCCESS;
    DBG_UTEST(HBPRINT("Add 3rd record...");)
    rc += hb_data_log_add_record(_p_test_log, test_str[2]);
    rc += hb_data_log_read_record(_p_test_log, rec_count, _buf);
    rc += hal_memcmp(_buf, test_str[2], _BUF_SIZE);
    _print_current_status(rc);
    rec_count++;
    rc_all += rc;

    rc = HB_SUCCESS;
    DBG_UTEST(HBPRINT("Resave 2nd record and add 4th record...");)
    rc += hb_data_log_save_record(_p_test_log, rec_count - 2, test_str[3]);
    rc += hb_data_log_add_record(_p_test_log, test_str[0]);

    rc += hb_data_log_read_record(_p_test_log, rec_count - 2, _buf);
    rc += hal_memcmp(_buf, test_str[3], _BUF_SIZE);
    rc += hb_data_log_read_record(_p_test_log, rec_count, _buf);
    rc += hal_memcmp(_buf, test_str[0], _BUF_SIZE);
    _print_current_status(rc);
    rec_count++;
    rc_all += rc;

    rc = HB_SUCCESS;
    DBG_UTEST(HBPRINT("Remove last record and read non-existent record...");)
    rc += hb_data_log_remove_last_record(_p_test_log);

    hal_memset(_buf, SOME_VALUE, _BUF_SIZE);
    rc += (hb_data_log_read_record(_p_test_log, rec_count, _buf) != HB_FAIL) ? HB_FAIL : 0;

    for (i = 0; i < _BUF_SIZE; i++) {
        if (_buf[i] != SOME_VALUE) {
            rc += HB_FAIL;
            break;
        }
    }
    _print_current_status(rc);
    rec_count--;
    rc_all += rc;

    *p_rec_count = rec_count;
    return rc_all;
}

uint32_t _test_count_and_size(uint32_t* p_rec_count)
{
    uint32_t rc = HB_SUCCESS, rc_all = HB_SUCCESS;
    uint32_t i = 0;
    uint32_t rec_count = *p_rec_count;

    rc = HB_SUCCESS;
    DBG_UTEST(HBPRINT("Check number of records...");)
    rc += (hb_data_log_get_num_records(_p_test_log) != rec_count) ? HB_FAIL : 0;
    for (i = 0; i < /*8*/400; i++) {
        rc += hb_data_log_add_record(_p_test_log, test_str[0]);
        rec_count++;
    }
    rc += hb_data_log_remove_last_record(_p_test_log);
    rec_count--;
    rc += (hb_data_log_get_num_records(_p_test_log) != rec_count) ? HB_FAIL : 0;
    _print_current_status(rc);
    rc_all += rc;

    rc = HB_SUCCESS;
    DBG_UTEST(HBPRINT("Check size of data...");)
    if (hb_data_log_get_size_of_data(_p_test_log) != (_p_test_log->rec_size * (min (hb_data_log_get_num_records(_p_test_log), _p_test_log->max_recs)))) {
        rc += HB_FAIL;
    }
    if (hb_data_log_get_size_of_data(_p_test_log) != (_p_test_log->rec_size * (min (rec_count, _p_test_log->max_recs)))) {
        rc += HB_FAIL;
    }
    _print_current_status(rc);
    rc_all += rc;

    *p_rec_count = rec_count;
    return rc_all;
}



uint32_t _test_read_raw_data_and_close(uint32_t* p_rec_count)
{
#define OFFSET_3TH_REC  (-(10 - 3 + 1))

    uint32_t rc = HB_SUCCESS, rc_all = HB_SUCCESS;
    uint32_t bytes_read = 0;
    uint32_t rec_count = *p_rec_count;

    DBG_UTEST(HBPRINT("Check reading data (3rd record) at opened .dat-file...");)
    hal_memset(_buf, SOME_VALUE, _BUF_SIZE);
    rc += hb_data_log_read_data(_p_test_log, ((rec_count + OFFSET_3TH_REC) % _p_test_log->max_recs) * _p_test_log->rec_size, _BUF_SIZE, _buf, &bytes_read);
    rc += (bytes_read != _BUF_SIZE) ? HB_FAIL : 0;
    rc += hal_memcmp(_buf, test_str[0], _BUF_SIZE);
    _print_current_status(rc);
    rc_all += rc;

    rc = HB_SUCCESS;
    DBG_UTEST(HBPRINT("Check closing .dat-file...\n");)
    rc += (rec_count != hb_data_log_get_num_records(_p_test_log)) ? HB_FAIL : 0;
    hb_data_log_close(_p_test_log);
    rc += (hb_data_log_add_record(_p_test_log, test_str[0]) == HB_SUCCESS) ? HB_FAIL : 0;
    rc += (rec_count != hb_data_log_get_num_records(_p_test_log)) ? HB_FAIL : 0;
    DBG_UTEST(HBPRINT("Check closing .dat-file is ");)
    _print_current_status(rc);
    rc_all += rc;

    rc = HB_SUCCESS;
    DBG_UTEST(HBPRINT("Check reading data (3rd record) at closed .dat-file ...\n");)
    hal_memset(_buf, SOME_VALUE, _BUF_SIZE);
    rc += hb_data_log_read_data(_p_test_log, ((rec_count + OFFSET_3TH_REC) % _p_test_log->max_recs) * _p_test_log->rec_size, _BUF_SIZE, _buf, &bytes_read);
    rc += (bytes_read != _BUF_SIZE) ? HB_FAIL : 0;
    rc += hal_memcmp(_buf, test_str[0], _BUF_SIZE);
    DBG_UTEST(HBPRINT("Check reading data at closed .dat-file is");)
    _print_current_status(rc);
    rc_all += rc;

    rc = HB_SUCCESS;
    DBG_UTEST(HBPRINT("Check closing .dat-file after reading...\n");)
    rc += (hb_data_log_add_record(_p_test_log, test_str[0]) == HB_SUCCESS) ? HB_FAIL : 0;
    DBG_UTEST(HBPRINT("Check closing .dat-file after reading is");)
    _print_current_status(rc);
    rc_all += rc;

    return rc_all;
}

uint32_t _check_Nth_cycle(void)
{
    uint32_t rc = HB_SUCCESS, rc_all = HB_SUCCESS;
    uint32_t rec_count = 0;
    uint8_t record[TEST_DATA_REC_SIZE];
    const int NCYCLES = 3;

    DBG_UTEST(HBPRINT("Init Test .dat-file...\n");)
    rc += _test_log_init();
    DBG_UTEST(HBPRINT("Init Test .dat-file is");)
    _print_current_status(rc);
    rc_all += rc;

    rc = HB_SUCCESS;
    for (int i = 0; i < NCYCLES * TEST_MAX_DATA_RECS_TO_STORE + 1; i++) {
        memset(record, 0, sizeof(record));
        *((uint32_t*)(record)) = rec_count;
        if (HB_SUCCESS == hb_data_log_add_record(_p_test_log, record)) {
            rec_count++;
        }
        else {
            rc++;
        }
    }
    DBG_UTEST(HBPRINT("Adding %i of %i records is ", rec_count, NCYCLES * TEST_MAX_DATA_RECS_TO_STORE);)
    _print_current_status(rc);
    rc_all += rc;

    if (rc) {
        return rc_all;
    }

    uint8_t *read_data = hal_malloc(TEST_DATA_REC_SIZE * TEST_MAX_DATA_RECS_TO_STORE);
    uint32_t bytes_read = 0;
    rc += hb_data_log_read_data(_p_test_log, 0, TEST_DATA_REC_SIZE * TEST_MAX_DATA_RECS_TO_STORE, read_data, &bytes_read);
    rc += (bytes_read != TEST_DATA_REC_SIZE * TEST_MAX_DATA_RECS_TO_STORE) ? HB_FAIL : 0;
    rc += *((uint32_t*)(read_data)) == NCYCLES * TEST_MAX_DATA_RECS_TO_STORE ? HB_SUCCESS : HB_FAIL;
    for (int i = (NCYCLES - 1) * TEST_MAX_DATA_RECS_TO_STORE + 1; i < NCYCLES * TEST_MAX_DATA_RECS_TO_STORE; i++) {
        rc += (0 == memcmp(&read_data[TEST_DATA_REC_SIZE * (i % TEST_MAX_DATA_RECS_TO_STORE)], &i, sizeof(i))) ? HB_SUCCESS : HB_FAIL;
    }
    hal_free(read_data);

    hb_data_log_close(_p_test_log);

    return rc;
}


uint32_t test_data_log_huge_records(void)
{
    uint32_t rc = HB_SUCCESS;
    uint32_t i = 0, rec_idx = 0, file_idx = 0;

    const int REC_SIZE = 200;
    const int MAX_RECS = 10000;
    const int MAX_FILES = 6;

    uint8_t* buf = alloca(REC_SIZE);

    for (i = 0; i < MAX_DATA_FILE_ID; i++)
    {
        *(uint32_t*)&_test_logs[i].rec_size = REC_SIZE;
        *(uint32_t*)&_test_logs[i].max_recs = MAX_RECS;
    }

    hb_dbg_setmask(0x4003000F);

    // delete .cfg-file for clear start
#if defined(HB_ITS_ONLY) || (defined(HB_WIN32) && defined(HB_UNIT_TESTS))
    hb_data_log_remove_cfg();
#endif
    hb_storage_uninit();
    hb_stor_format_drive(0);
    hb_storage_init();

    DBG_UTEST(HBPRINT("\nInit Data Log...\n");)
    rc += hb_data_log_init();
    DBG_UTEST(HBPRINT("Init Data Log is");)
    _print_current_status(rc);

    for (file_idx = 0; file_idx < MAX_FILES; file_idx++) {
        hb_data_log_create(&_test_logs[file_idx]);
    }
    hal_feed_the_dog();

    DBG_UTEST(HBPRINT("\nCheck write read big amount of records:");)
    DBG_UTEST(HBPRINT("\n*********************************************************************\n");)

    for (rec_idx = 0; rec_idx < 10 * MAX_RECS; rec_idx++)
    {
        for (file_idx = 0; file_idx < MAX_FILES; file_idx++)
        {
            _p_test_log = &_test_logs[file_idx];

            *(uint32_t*)buf = file_idx * rec_idx;
            hb_data_log_add_record(_p_test_log, buf);

            hb_data_log_read_record(_p_test_log, rec_idx, buf);
            if (*(uint32_t*)buf != file_idx * rec_idx) {
                DBG_UTEST(HBPRINT("error of reading %ld record in %ld file (read %ld, expected %ld)\n", rec_idx, file_idx, *(uint32_t*)buf, file_idx * rec_idx);)
                rc = HB_FAIL;
            }
        }
        hal_feed_the_dog();
        if (rec_idx % 100 == 0) {
            DBG_UTEST(HBPRINT("progress: rec_idx = %ld\n", rec_idx, file_idx);)
        }
    }

    DBG_UTEST(HBPRINT("\n*********************************************************************\n");)
    DBG_UTEST(HBPRINT("Check write read big amount of records ... ");)
    _print_current_status(rc);
//     if (rc) {
//         return HB_FAIL;
//     }

    DBG_UTEST(HBPRINT("\nCheck read big amount of records:");)
    DBG_UTEST(HBPRINT("\n*********************************************************************\n");)

    for (rec_idx = 0; rec_idx < 10 * MAX_RECS; rec_idx++)
    {
        for (file_idx = 0; file_idx < MAX_FILES; file_idx++)
        {
            _p_test_log = &_test_logs[file_idx];

            *(uint32_t*)buf = file_idx * rec_idx;

            hb_data_log_read_record(_p_test_log, rec_idx, buf);
            if (*(uint32_t*)buf != file_idx * rec_idx) {
                DBG_UTEST(HBPRINT("error of reading %ld record in %ld file (read %ld, expected %ld)\n", rec_idx, file_idx, *(uint32_t*)buf, file_idx * rec_idx);)
                rc = HB_FAIL;
            }
        }
        hal_feed_the_dog();
        if (rec_idx % 100 == 0) {
            DBG_UTEST(HBPRINT("progress: rec_idx = %ld\n", rec_idx, file_idx);)
        }
    }
    DBG_UTEST(HBPRINT("\n*********************************************************************\n");)
    DBG_UTEST(HBPRINT("Check read big amount of records ... ");)
    _print_current_status(rc);

    hb_data_log_uninit();

    DBG_UTEST(HBPRINT("\n");)

    return rc;
}


void _print_current_status(uint32_t status)
{
    if (status == HB_SUCCESS) {
        DBG_UTEST(HBPRINT(" OK!\n");)
    } else {
        DBG_UTEST(HBPRINT(" FAILED! \n");)
    }
}
/* ================================================================================= */
#endif // HB_UNIT_TESTS
