 /***********************************************************************\
 * Copyright (c) 2013 by  LLC. All Rights Reserved.                *
 *                                                                       *
 * This is the header file for  Core Storage                       *
 \***********************************************************************/

/*
 * HISTORY:
 *
 *    Rev. 1.00 (03/21/13) AK: first release
 *
 */


#ifndef HB_STORAGE_H
#define HB_STORAGE_H

#include "os_dependency.h"          /* platform dependent definitions and includes */


/** Definition of File System. */
#if defined(HB_SENSOR_v3_BOARD) || defined(HB_SENSOR_v3S_BOARD) || defined(HB_SENSOR_v3T_BOARD)
#define FAT_FS
#elif defined(HB_SENSOR_v4L_BOARD) || defined(HB_SENSOR_v4_BOARD)
#define LITTLE_FS
#endif // HB_SENSOR_xxx_BOARD


/*************************************************************************
*  Functions Implemented by this module
*************************************************************************/

#define HB_STOR_ERR_INCOMPLATE_WRITE    20
#define HB_STOR_ERR_NO_FILE             4

#define SHORT_FILE_NAME_MAX 13

typedef struct
{
    uint8_t fattrib;                // Attribute
    uint32_t size;                  // File size
    char name[SHORT_FILE_NAME_MAX]; // Short file name (8.3 format)

} file_info_t;

typedef struct
{
    void*    handle;
    uint32_t idx;
    uint32_t recs_count;     // AK TODO: transform this into 'current offset' to reduce seeking
} f_handle_t;


uint32_t hb_storage_init(void);
uint32_t hb_storage_uninit(void);
uint32_t hb_stor_format_drive(HB_BOOL init);

uint32_t hb_stor_open_file(f_handle_t* phandle, const char* pfname);
uint32_t hb_stor_seek_file(f_handle_t* phandle, const uint32_t offset); // Don`t use this, use "hb_stor_set_rw_pos" instead 
uint32_t hb_stor_write_file(f_handle_t* phandle, uint8_t* pbuf, uint32_t size, uint32_t* pbw); // Don`t use this, use "hb_stor_write_buf" instead
uint32_t hb_stor_read_file(f_handle_t* phandle, uint8_t* pbuf, uint32_t size, uint32_t* pbr); // Don`t use this, use "hb_stor_read_buf" instead
uint32_t hb_stor_clear_file(f_handle_t* phandle);
uint32_t hb_stor_close_file(f_handle_t* phandle);
uint32_t hb_stor_delete_file(const char* pfname);
uint32_t hb_stor_tell_file(f_handle_t* phandle);
uint32_t hb_stor_sync_file(f_handle_t* phandle);
int32_t hb_stor_size_file(f_handle_t* phandle); // Don`t use this, use "hb_stor_get_file_size" instead
uint32_t hb_stor_trunc_file(f_handle_t* phandle, const uint32_t size);

uint32_t hb_stor_get_file_info(const char *path, file_info_t *fno);
uint32_t hb_stor_rename(const char *path_old, const char *path_new);

uint32_t hb_stor_create_log_file(f_handle_t* phandle, const char* fname, uint32_t size);

uint32_t hb_stor_create_file(f_handle_t* phandle, const char* fname);

uint32_t hb_stor_wipe_file(f_handle_t* phandle, uint32_t offset, uint32_t size);

uint32_t hb_stor_file_is_opened(f_handle_t* phandle);

uint32_t hb_stor_read_buf(f_handle_t* phandle, void* pbuf, uint32_t bytes);
uint32_t hb_stor_read_buf_from_offset(f_handle_t* phandle, void* pbuf, uint32_t bytes, uint32_t offset);
uint32_t hb_stor_write_buf(f_handle_t* phandle, void* pbuf, uint32_t bytes);
uint32_t hb_stor_set_rw_pos(f_handle_t* phandle, uint32_t offset);

int32_t  hb_stor_get_file_size(f_handle_t* phandle);

// Reads file contents, starting from 'start_address', 'bytes' length into 'buffer'
// if file is already opened, 'phandle' must be provided, otherwise 'fname' is a file name
uint32_t hb_stor_read_log_portion(f_handle_t *phandle, const char *fname, uint32_t start_address, uint32_t bytes, void *buffer, uint32_t *bytes_read);

//Gets records count from 'cfg' file
//if file is already opened, 'phandle' must be provided, otherwise 'fname' is a file name
uint32_t hb_stor_get_records_count(f_handle_t *pDat_handle, f_handle_t *pCfg_handle, const char *dat_fname, const char *cfg_fname);

int32_t hb_stor_get_error_code(void);
/* ==================================================================== */
#endif // HB_STORAGE_H
