/*
 * Copyright (c) 2012  LLC. All Rights Reserved
 *
 * Platform/OS dependent definitions for HB projects (alter as needed for your platform).
 *
 * Rev. 2013-03
 *
 */


#if !defined(__HB_OS_DEPENDENT_TYPEDEFS__)
#define __HB_OS_DEPENDENT_TYPEDEFS__


/*
 * Comment out the following platform definitions except for the OS that
 * the associated project is developed for...
 *
 */

//#undef _WIN32
//#define _WIN32

#if defined(_WIN32) || defined(__linux__)
#define HB_WIN32
#else
#define HB_ATMEL_ARM
#endif


/*************************************************************************
*  Sensor Firmware Compilation options
*************************************************************************/
//#include "hb_build_config.h"
/* ==================================================================== */


/*************************************************************************
*  Common definitions
*************************************************************************/
#define HB_ERROR        -1
#define HB_SUCCESS       0
#define HB_FAIL          1
#define HB_PENDING       2
#define HB_NOT_SUPPORTED 3

#define HB_FALSE        0
#define HB_TRUE         1

#define HB_MALE         1
#define HB_FEMALE       2

#define DISABLE         0
#define ENABLE          1

/******** DSP_lib related *************/
#define Q7_T_MAX   127
#define Q7_T_MIN   (-128)

#define Q15_T_MAX   32767
#define Q15_T_MIN   (-32768)

#define Q31_T_MAX   2147483647
#define Q31_T_MIN   (-2147483647 - 1)
/**************************************/
/* ==================================================================== */


/* OS-specific definitions */

#ifdef HB_WIN32

    #undef HB_ATMEL_ARM

    /***** Calling conventions ************/

    #define HB_PREFIX      extern
    #define HB_INLINE    __inline
    #if defined(_WIN32)
    #define HB_PACKED    __declspec(align(1))
    #else
    #define HB_PACKED
    #endif
    #define HB_REGISTER    register

	#ifndef __GNUC__
    #define __attribute__(x)
	#endif

    /***** Platform specific includes *****/

    #include <stdio.h>
    #include <stdlib.h>
    #include <time.h>
    #include <memory.h>
    #if defined(_WIN32)
    #include <windows.h>
    #endif
    #include <math.h>
    #include <stddef.h>      // offsetof()


    #define hal_memset memset
    #define hal_memcpy memcpy
    #define hal_memcmp memcmp

    #define hal_malloc malloc
    #define hal_free   free

    /***** Platform specific typedefs *****/

    typedef unsigned char   uint8_t;
    typedef unsigned short  uint16_t;
    typedef unsigned int    uint32_t;
//    typedef unsigned long long uint64_t;

    typedef unsigned char   Bool;  // for FS's sysdata

#ifndef __linux__
    typedef signed char     int8_t;
    typedef signed short    int16_t;
    typedef signed int      int32_t;
#endif

    typedef float           float32_t;
    typedef float           HB_FLOAT;
    typedef double          HB_DOUBLE;

    //typedef time_t          HB_TIME;
    typedef int32_t         HB_TIME;    // for build environments, where time_t is 8 bytes

    typedef int32_t         HB_BOOL;
    /***** Platform specific definitions **/

    #ifndef NULL
      #define NULL 0
    #endif // NULL

    #define HB_SYS_TICK_MS 10


    /***** Debug stuff ********************/
#ifdef GUIDEMO
    extern void PrintToDataRetriever(const char *fmt, ...);
    #define HBPRINT(...) PrintToDataRetriever(__VA_ARGS__)

    typedef int16_t         q15_t;
#else
//     #define HBPRINT   printf
//     #define HBFPRINTF printf
    #define HBPRINT(q, ...) printf(q, ##__VA_ARGS__)
    //#define HBFPRINTF(q, ...) hb_fprintf(q, ##__VA_ARGS__)
    #define HBFPRINTF(fmt, ...) do {                            \
        printf(fmt, ##__VA_ARGS__);                             \
    } while (0);
    //#define FPRINTF_PRINTS_TO_FILE
#endif

    /**************************************/

    /******** DSP_lib related *************/
    //     #define WINDOWS_ENABLE_CPLUSPLUS
    // #include <basetsd.h>
    // #include <limits.h>
    //     #define __INLINE        __inline
    //     #define __STATIC_INLINE static __inline
    //     #define __packed        __attribute__ ((packed))
        //#define INT32_MIN     INT_MIN
        //#define INT32_MAX     INT_MAX

    //     typedef INT64           int64_t;

    // #define WIN32_ARM_EMULATION     // define this on the project level (as a preprocessor macro)

    #include "float.h"

#if !defined(HB_BASE_FW)
//#include "arm_math.h"
    #include "hb_stdint.h"
#endif
    /**************************************/

    /* a portable snprintf implementation */
    #if __cplusplus < 201103L
//#define snprintf _snprintf
    #endif
    /**************************************/
#endif // HB_WIN32




#ifdef HB_ATMEL_ARM

    #undef HB_WIN32
    #undef HB_ITS_ONLY       // reinsurance

    /***** Calling conventions ************/

    #define HB_PREFIX      extern
    #define HB_INLINE      static inline
    //#define HB_PACKED      __attribute__ ((aligned(1), packed))
    #define HB_PACKED      __attribute__ ((packed))
    #define HB_REGISTER    register


    /***** Platform specific includes *****/

    #include <asf.h>
    #include <stdio.h>
    #include <math.h>

    #include <stdlib.h>          // malloc
    #include <string.h>          // memset, etc.

    #define hal_memset memset
    #define hal_memcpy memcpy
    #define hal_memcmp memcmp

    //#define hal_malloc malloc
    #define hal_free   free

    /***** Platform specific typedefs *****/

    typedef unsigned char   uint8_t;
    typedef unsigned short  uint16_t;
    typedef unsigned long   uint32_t;

    typedef signed char     int8_t;
    typedef signed short    int16_t;
    typedef signed long     int32_t;

    typedef unsigned long long uint64_t;
    typedef signed long long int64_t;

    typedef unsigned char   Bool;  // for FS's sysdata


    typedef float           float32_t;
    typedef float           HB_FLOAT;
    typedef double          HB_DOUBLE;

    typedef int32_t         HB_TIME;

    typedef int32_t         HB_BOOL;

    /***** Platform specific definitions **/

    #ifndef NULL
      #define NULL 0
    #endif // NULL

#if (defined(HB_BASE_FW) || defined(HB_TEST))
    #define HB_SYS_TICK_MS 1
#else
    #define HB_SYS_TICK_MS 5
    //#define HB_SYS_TICK_MS 20
#endif

    /************************************************************************/
    /*                min and max macro defines                             */
    /************************************************************************/
    #ifdef max
        #undef max
    #endif
    #ifdef min
        #undef min
    #endif

    #define max(a,b) \
        ({ __typeof__ (a) _a = (a); \
            __typeof__ (b) _b = (b); \
            _a > _b ? _a : _b; })

    #define min(a,b) \
        ({ __typeof__ (a) _a1 = (a); \
            __typeof__ (b) _b1 = (b); \
            _a1 < _b1 ? _a1 : _b1; })

    /***** Debug stuff ********************/

    #ifdef HB_DEBUG
    //#define HBPRINT printf
    #define HBPRINT(q,...) printf(q,##__VA_ARGS__)
    #define HBFPRINTF HBPRINT
    //#define FPRINTF_PRINTS_TO_FILE
    #else
    #define HBPRINT(...)
    #define HBFPRINTF(...)
    #endif

    /**************************************/

    #include "arm_math.h"

#endif // HB_ATMEL_ARM


#define STR_HELPER(x) #x
#define STR(x) STR_HELPER(x)

#define CAT_(a, b, c, d) a ## b ## c ## d
#define CAT(a, b, c, d) CAT_(a, b, c, d)
// a warningless version of static assert, see http://www.pixelbeat.org/programming/gcc/static_assert.html
#define STATIC_ASSERT(e); enum {CAT(static_assert_, __COUNTER__, _at_line_, __LINE__) = 1/(int)(!!(e))}

#define STATIC_ASSERT_TYPE_SIZE(name, size) typedef char wrong_size_of_type_##name [(sizeof(name) == (size))?1:-1]
#define STATIC_ASSERT_DIVIZIBLE_BY_4(name) typedef char not_divisible_by_4_##name [(sizeof(name)%4 == 0)?1:-1]

#include "Utils/hb_trace.h"
#include "Utils/hb_assert.h"

/* taking care of required '#define's */

#ifndef HB_PACKED
#error "Need to define attribute HB_PACKED even if the definition is blank"
#endif

#endif // __HB_OS_DEPENDENT_TYPEDEFS__

/** (END OF FILE OS_dependency.h) **************************************************/
